@@ -11,6 +11,9 @@ class ActionPackAssertionsController < ActionController::Base
 
   # a standard template
   def hello_xml_world() render :template => "test/hello_xml_world"; end
+  
+  # a standard partial
+  def partial() render :partial => 'test/partial'; end
 
   # a redirect to an internal location
   def redirect_internal() redirect_to "/nothing"; end
@@ -137,6 +140,9 @@ class AssertResponseWithUnexpectedErrorController < ActionController::Base
   end
 end
 
+class UserController < ActionController::Base
+end
+
 module Admin
   class InnerModuleController < ActionController::Base
     def index
@@ -154,29 +160,19 @@ module Admin
     def redirect_to_fellow_controller
       redirect_to :controller => 'user'
     end
-    
+
     def redirect_to_top_level_named_route
       redirect_to top_level_url(:id => "foo")
     end
   end
 end
 
-# ---------------------------------------------------------------------------
-
-
-# tell the controller where to find its templates but start from parent
-# directory of test_request_response to simulate the behaviour of a
-# production environment
-ActionPackAssertionsController.view_paths = [ File.dirname(__FILE__) + "/../fixtures/" ]
-
 # a test case to exercise the new capabilities TestRequest & TestResponse
-class ActionPackAssertionsControllerTest < Test::Unit::TestCase
+class ActionPackAssertionsControllerTest < ActionController::TestCase
   # let's get this party started
   def setup
     ActionController::Routing::Routes.reload
-    ActionController::Routing.use_controllers!(%w(action_pack_assertions admin/inner_module content admin/user))
-    @controller = ActionPackAssertionsController.new
-    @request, @response = ActionController::TestRequest.new, ActionController::TestResponse.new
+    ActionController::Routing.use_controllers!(%w(action_pack_assertions admin/inner_module user content admin/user))
   end
 
   def teardown
@@ -229,7 +225,6 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
       process :redirect_to_named_route
       assert_redirected_to 'http://test.host/route_one'
       assert_redirected_to route_one_url
-      assert_redirected_to :route_one_url
     end
   end
 
@@ -241,18 +236,15 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
         map.connect   ':controller/:action/:id'
       end
       process :redirect_to_named_route
-      assert_raise(Test::Unit::AssertionFailedError) do
+      assert_raise(ActiveSupport::TestCase::Assertion) do
         assert_redirected_to 'http://test.host/route_two'
       end
-      assert_raise(Test::Unit::AssertionFailedError) do
+      assert_raise(ActiveSupport::TestCase::Assertion) do
         assert_redirected_to :controller => 'action_pack_assertions', :action => 'nothing', :id => 'two'
       end
-      assert_raise(Test::Unit::AssertionFailedError) do
+      assert_raise(ActiveSupport::TestCase::Assertion) do
         assert_redirected_to route_two_url
       end
-      assert_raise(Test::Unit::AssertionFailedError) do
-        assert_redirected_to :route_two_url
-      end
     end
   end
 
@@ -268,7 +260,7 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
       assert_redirected_to admin_inner_module_path
     end
   end
-  
+
   def test_assert_redirected_to_top_level_named_route_from_nested_controller
     with_routing do |set|
       set.draw do |map|
@@ -277,11 +269,25 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
       end
       @controller = Admin::InnerModuleController.new
       process :redirect_to_top_level_named_route
-      # passes -> assert_redirected_to "http://test.host/action_pack_assertions/foo"
+      # assert_redirected_to "http://test.host/action_pack_assertions/foo" would pass because of exact match early return
       assert_redirected_to "/action_pack_assertions/foo"
     end
   end
 
+  def test_assert_redirected_to_top_level_named_route_with_same_controller_name_in_both_namespaces
+    with_routing do |set|
+      set.draw do |map|
+        # this controller exists in the admin namespace as well which is the only difference from previous test
+        map.top_level '/user/:id', :controller => 'user', :action => 'index'
+        map.connect   ':controller/:action/:id'
+      end
+      @controller = Admin::InnerModuleController.new
+      process :redirect_to_top_level_named_route
+      # assert_redirected_to top_level_url('foo') would pass because of exact match early return
+      assert_redirected_to top_level_path('foo')
+    end
+  end
+
   # -- standard request/response object testing --------------------------------
 
   # make sure that the template objects exist
@@ -323,11 +329,35 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
   # check if we were rendered by a file-based template?
   def test_rendered_action
     process :nothing
-    assert !@response.rendered_with_file?
+    assert_nil @response.rendered[:template]
 
     process :hello_world
-    assert @response.rendered_with_file?
-    assert 'hello_world', @response.rendered_file
+    assert @response.rendered[:template]
+    assert 'hello_world', @response.rendered[:template].to_s
+  end
+  
+  def test_assert_template_with_partial
+    get :partial
+    assert_template :partial => '_partial'
+  end
+  
+  def test_assert_template_with_nil
+    get :nothing
+    assert_template nil
+  end
+  
+  def test_assert_template_with_string
+    get :hello_world
+    assert_template 'hello_world'    
+  end
+  
+  def test_assert_template_with_symbol
+    get :hello_world
+    assert_template :hello_world
+  end
+  
+  def test_assert_template_with_bad_argument
+    assert_raise(ArgumentError) { assert_template 1 }    
   end
 
   # check the redirection location
@@ -363,6 +393,12 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
     assert @response.missing?
   end
 
+  # check client errors
+  def test_client_error_response_code
+    process :response404
+    assert @response.client_error?
+  end
+
   # check to see if our redirection matches a pattern
   def test_redirect_url_match
     process :redirect_external
@@ -402,26 +438,10 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
     assert_equal "Mr. David", @response.body
   end
 
-  def test_follow_redirect
-    process :redirect_to_action
-    assert_redirected_to :action => "flash_me"
-
-    follow_redirect
-    assert_equal 1, @request.parameters["id"].to_i
-
-    assert "Inconceivable!", @response.body
-  end
-
-  def test_follow_redirect_outside_current_action
-    process :redirect_to_controller
-    assert_redirected_to :controller => "elsewhere", :action => "flash_me"
-
-    assert_raises(RuntimeError, "Can't follow redirects outside of current controller (elsewhere)") { follow_redirect }
-  end
 
   def test_assert_redirection_fails_with_incorrect_controller
     process :redirect_to_controller
-    assert_raise(Test::Unit::AssertionFailedError) do
+    assert_raise(ActiveSupport::TestCase::Assertion) do
       assert_redirected_to :controller => "action_pack_assertions", :action => "flash_me"
     end
   end
@@ -431,14 +451,16 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
     assert_redirected_to :controller => 'action_pack_assertions', :action => "flash_me", :id => 1, :params => { :panda => 'fun' }
   end
 
-  def test_redirected_to_url_leadling_slash
+  def test_redirected_to_url_leading_slash
     process :redirect_to_path
     assert_redirected_to '/some/path'
   end
+
   def test_redirected_to_url_no_leadling_slash
     process :redirect_to_path
     assert_redirected_to 'some/path'
   end
+
   def test_redirected_to_url_full_url
     process :redirect_to_path
     assert_redirected_to 'http://test.host/some/path'
@@ -458,7 +480,7 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
   def test_redirected_to_with_nested_controller
     @controller = Admin::InnerModuleController.new
     get :redirect_to_absolute_controller
-    assert_redirected_to :controller => 'content'
+    assert_redirected_to :controller => '/content'
 
     get :redirect_to_fellow_controller
     assert_redirected_to :controller => 'admin/user'
@@ -466,16 +488,16 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
 
   def test_assert_valid
     get :get_valid_record
-    assert_valid assigns('record')
+    assert_deprecated { assert_valid assigns('record') }
   end
 
   def test_assert_valid_failing
     get :get_invalid_record
 
     begin
-      assert_valid assigns('record')
+      assert_deprecated { assert_valid assigns('record') }
       assert false
-    rescue Test::Unit::AssertionFailedError => e
+    rescue ActiveSupport::TestCase::Assertion => e
     end
   end
 
@@ -484,7 +506,7 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
     get :index
     assert_response :success
     flunk 'Expected non-success response'
-  rescue Test::Unit::AssertionFailedError => e
+  rescue ActiveSupport::TestCase::Assertion => e
     assert e.message.include?('FAIL')
   end
 
@@ -493,32 +515,29 @@ class ActionPackAssertionsControllerTest < Test::Unit::TestCase
     get :show
     assert_response :success
     flunk 'Expected non-success response'
-  rescue Test::Unit::AssertionFailedError
+  rescue ActiveSupport::TestCase::Assertion
+    # success
   rescue
     flunk "assert_response failed to handle failure response with missing, but optional, exception."
   end
 end
 
-class ActionPackHeaderTest < Test::Unit::TestCase
-  def setup
-    @controller = ActionPackAssertionsController.new
-    @request, @response = ActionController::TestRequest.new, ActionController::TestResponse.new
-  end
+class ActionPackHeaderTest < ActionController::TestCase
+  tests ActionPackAssertionsController
 
   def test_rendering_xml_sets_content_type
     process :hello_xml_world
-    assert_equal('application/xml; charset=utf-8', @response.headers['type'])
+    assert_equal('application/xml; charset=utf-8', @response.headers['Content-Type'])
   end
 
   def test_rendering_xml_respects_content_type
     @response.headers['type'] = 'application/pdf'
     process :hello_xml_world
-    assert_equal('application/pdf; charset=utf-8', @response.headers['type'])
+    assert_equal('application/pdf; charset=utf-8', @response.headers['Content-Type'])
   end
 
-
   def test_render_text_with_custom_content_type
     get :render_text_with_custom_content_type
-    assert_equal 'application/rss+xml; charset=utf-8', @response.headers['type']
+    assert_equal 'application/rss+xml; charset=utf-8', @response.headers['Content-Type']
   end
 end
