@@ -1,4 +1,4 @@
-require File.expand_path(File.join(File.dirname(__FILE__), '..', 'test_helper'))
+require 'test_helper'
 
 class MachineTestSubject
   include ActiveModel::StateMachine
@@ -40,4 +40,4 @@ class StateMachineMachineTest < ActiveModel::TestCase
     assert events.include?(:shutdown)
     assert events.include?(:timeout)
   end
-end
\ No newline at end of file
+end
