@@ -1,4 +1,6 @@
+require 'active_support/core_ext/object/blank'
 require 'active_support/core_ext/object/conversions'
 require 'active_support/core_ext/object/extending'
 require 'active_support/core_ext/object/instance_variables'
+require 'active_support/core_ext/object/metaclass'
 require 'active_support/core_ext/object/misc'
