@@ -2,10 +2,10 @@ require 'abstract_unit'
 
 class TestMissingSourceFile < Test::Unit::TestCase
   def test_with_require
-    assert_raises(MissingSourceFile) { require 'no_this_file_don\'t_exist' }
+    assert_raise(MissingSourceFile) { require 'no_this_file_don\'t_exist' }
   end
   def test_with_load
-    assert_raises(MissingSourceFile) { load 'nor_does_this_one' }
+    assert_raise(MissingSourceFile) { load 'nor_does_this_one' }
   end
   def test_path
     begin load 'nor/this/one.rb'
