@@ -145,6 +145,19 @@ class GeneratorTestCase < Test::Unit::TestCase
     end
   end
 
+  # Asserts that the given helper test test was generated.
+  # It takes a name or symbol without the <tt>_helper_test</tt> part and an optional super class.
+  # The contents of the class source file is passed to a block.
+  def assert_generated_helper_test_for(name, parent = "ActionView::TestCase")
+    path = "test/unit/helpers/#{name.to_s.underscore}_helper_test"
+    # Have to pass the path without the "test/" part so that class_name_from_path will return a correct result
+    class_name = class_name_from_path(path.gsub(/^test\//, ''))
+
+    assert_generated_class path,parent,class_name do |body|
+      yield body if block_given?
+    end
+  end
+
   # Asserts that the given unit test was generated.
   # It takes a name or symbol without the <tt>_test</tt> part and an optional super class.
   # The contents of the class source file is passed to a block.
@@ -172,22 +185,24 @@ class GeneratorTestCase < Test::Unit::TestCase
   # Asserts that the given class source file was generated.
   # It takes a path without the <tt>.rb</tt> part and an optional super class.
   # The contents of the class source file is passed to a block.
-  def assert_generated_class(path, parent = nil)
-    # FIXME: Sucky way to detect namespaced classes
-    if path.split('/').size > 3
-      path =~ /\/?(\d+_)?(\w+)\/(\w+)$/
-      class_name = "#{$2.camelize}::#{$3.camelize}"
-    else
-      path =~ /\/?(\d+_)?(\w+)$/
-      class_name = $2.camelize
-    end
-
+  def assert_generated_class(path, parent = nil, class_name = class_name_from_path(path))
     assert_generated_file("#{path}.rb") do |body|
       assert_match /class #{class_name}#{parent.nil? ? '':" < #{parent}"}/, body, "the file '#{path}.rb' should be a class"
       yield body if block_given?
     end
   end
 
+  def class_name_from_path(path)
+    # FIXME: Sucky way to detect namespaced classes
+    if path.split('/').size > 3
+      path =~ /\/?(\d+_)?(\w+)\/(\w+)$/
+      "#{$2.camelize}::#{$3.camelize}"
+    else
+      path =~ /\/?(\d+_)?(\w+)$/
+      $2.camelize
+    end
+  end
+
   # Asserts that the given module source file was generated.
   # It takes a path without the <tt>.rb</tt> part.
   # The contents of the class source file is passed to a block.
@@ -249,6 +264,8 @@ class GeneratorTestCase < Test::Unit::TestCase
 
   def assert_generated_migration(name, parent = "ActiveRecord::Migration")
     file = Dir.glob("#{RAILS_ROOT}/db/migrate/*_#{name.to_s.underscore}.rb").first
+    assert !file.nil?, "should have generated the migration file but didn't"
+
     file = file.match(/db\/migrate\/[0-9]+_\w+/).to_s
     assert_generated_class file, parent do |body|
       assert_match /timestamps/, body, "should have timestamps defined"
@@ -285,4 +302,9 @@ class GeneratorTestCase < Test::Unit::TestCase
   def assert_generated_column(body, name, type)
     assert_match /t\.#{type.to_s} :#{name.to_s}/, body, "should have column #{name.to_s} defined"
   end
+
+  # Asserts that the given table is defined in the migration.
+  def assert_generated_table(body, name)
+    assert_match /create_table :#{name.to_s} do/, body, "should have table #{name.to_s} defined"
+  end
 end
